#!/usr/bin/python
# encoding: utf-8
#
# Copyright (c) 2019 Dean Jackson <deanishe@deanishe.net>
#
# MIT Licence. See http://opensource.org/licenses/MIT
#
# Created on 2019-05-30
#

"""Alfred 3+ Script Filter to provide web searches via suffix keywords."""

from __future__ import print_function, absolute_import

import json
import os
from plistlib import readPlist
import sys
from urllib import quote, quote_plus

# Path to Alfred's preferences bundle
PREFS = os.getenv('alfred_preferences')

# Whether Alfred's debugger is open
DEBUG = os.getenv('alfred_debug') == '1'


def log(s, *args):
    """Log to STDERR."""
    if not DEBUG:
        return
    if args:
        s = s % args
    print(s, file=sys.stderr)


def decode(v):
    """Decode str to Unicode."""
    if isinstance(v, str):
        return v.decode('utf-8')
    if isinstance(v, dict):
        for k in v:
            v[k] = decode(v[k])
    return v


def show_error(err):
    """Show an error in Alfred."""
    title = str(err)
    icon = ('/System/Library/CoreServices/CoreTypes.bundle/'
            'Contents/Resources/AlertStopIcon.icns')
    write_items({
        'title': title,
        'icon': {'path': icon},
        'valid': False,
    })


def warn_item(title, subtitle=u''):
    """Alfred item with warn icon."""
    icon = ('/System/Library/CoreServices/CoreTypes.bundle/'
            'Contents/Resources/AlertCautionIcon.icns')
    return {
        'title': title,
        'subtitle': subtitle,
        'icon': {'path': icon},
        'valid': False,
    }


def write_items(*items):
    """Send JSON feedback to Alfred."""
    json.dump(dict(items=items), sys.stdout, indent=2, sort_keys=True)


def config_path():
    """Return path to Web Search `prefs.plist`."""
    return os.path.join(PREFS, 'preferences/features/websearch/prefs.plist')


class Search(object):
    """An Alfred Web Search."""

    def __init__(self, uid, keyword, url, text, enabled, utf8, plusSpaces=None,
                 spaces=None):
        """Create a new `Search`."""
        self.uid = uid
        self.keyword = keyword
        self.url = url
        self.text = text
        self.enabled = enabled
        self.utf8 = utf8
        if spaces == '+':
            self.quote = quote_plus
        else:
            self.quote = quote

    def __call__(self, query):
        """Return Alfred `item` dict."""
        return dict(
            title=self.text.format(query=query),
            subtitle='Keyword: ' + self.keyword,
            valid=True,
            arg=self.url.format(query=self.quote(query.encode('utf-8'))),
            uid=self.uid,
            icon=dict(path=self.icon or 'icon.png'),
        )

    def match(self, keyword):
        """Return `True` if `keyword` is prefix of Search keyword."""
        return self.keyword.startswith(keyword)

    @property
    def icon(self):
        p = os.path.join(PREFS, 'resources',
                         'features.websearch.custom.' + self.uid + '.png')
        if os.path.exists(p):
            return p


def load_searches(path):
    """Parse `prefs.plist` file and return Searches in it."""
    searches = []
    log('loading searches from %r ...', path)
    data = readPlist(path)
    for uid, d in data.get('customSites', {}).items():
        if not d['enabled']:
            continue
        d['uid'] = uid
        d = decode(d)

        # log('search=%r', d)
        searches.append(Search(**d))

    return searches


def parse_query(query):
    """Parse query into query and matching searches."""
    searches = load_searches(config_path())
    searches = [s for s in searches if s.enabled]
    log('%d enabled search(es)', len(searches))

    words = query.split()
    if len(words) < 2:
        return query.strip(), searches

    last = words.pop().strip()
    matches = [s for s in searches if s.match(last)]
    log('%d search(es) match %r', len(matches), last)
    if matches:
        return ' '.join(words), matches

    return query, searches


def main():
    """Run Script Filter."""
    items = []
    query = sys.argv[1].decode('utf-8')
    log('.\nquery=%r', query)
    query, searches = parse_query(query)
    if not searches:
        write_items(warn_item('No Searches Enabled',
                              'You must enable some web searches'))
        return

    for s in searches:
        items.append(s(query))

    write_items(*items)


if __name__ == "__main__":
    try:
        sys.exit(main())
    except Exception as err:
        from traceback import print_exc
        print_exc(file=sys.stderr)
        show_error(err)
        raise err
